/* A lexical scanner generated by flex */

/* Scanner skeleton version:
 * $Header: /home/daffy/u0/vern/flex/RCS/flex.skl,v 2.91 96/09/10 16:58:48 vern Exp $
 */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5

#include <stdio.h>


/* cfront 1.2 defines "c_plusplus" instead of "__cplusplus" */
#ifdef c_plusplus
#ifndef __cplusplus
#define __cplusplus
#endif
#endif


#ifdef __cplusplus

#include <stdlib.h>
#include <unistd.h>

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#if __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */

#ifdef __TURBOC__
 #pragma warn -rch
 #pragma warn -use
#include <io.h>
#include <stdlib.h>
#define YY_USE_CONST
#define YY_USE_PROTOS
#endif

#ifdef YY_USE_CONST
#define __IDL_const const
#else
#define __IDL_const
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN __IDL__start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((__IDL__start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE __IDL_restart( __IDL_in )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct __IDL__buffer_state *YY_BUFFER_STATE;

extern int __IDL_leng;
extern FILE *__IDL_in, *__IDL_out;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 * 	if ( condition_holds )
 *		__IDL_less( 5 );
 *	else
 *		do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the __IDL_less() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define __IDL_less(n) \
	do \
		{ \
		/* Undo effects of setting up __IDL_text. */ \
		*__IDL__cp = __IDL__hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		__IDL__c_buf_p = __IDL__cp = __IDL__bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up __IDL_text again */ \
		} \
	while ( 0 )

#define unput(c) __IDL_unput( c, __IDL_text_ptr )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */
typedef unsigned int __IDL__size_t;


struct __IDL__buffer_state
	{
	FILE *__IDL__input_file;

	char *__IDL__ch_buf;		/* input buffer */
	char *__IDL__buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	__IDL__size_t __IDL__buf_size;

	/* Number of characters read into __IDL__ch_buf, not including EOB
	 * characters.
	 */
	int __IDL__n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int __IDL__is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int __IDL__is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int __IDL__at_bol;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int __IDL__fill_buffer;

	int __IDL__buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via __IDL_restart()), so that the user can continue scanning by
	 * just pointing __IDL_in at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};

static YY_BUFFER_STATE __IDL__current_buffer = 0;

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER __IDL__current_buffer


/* __IDL__hold_char holds the character lost when __IDL_text is formed. */
static char __IDL__hold_char;

static int __IDL__n_chars;		/* number of characters read into __IDL__ch_buf */


int __IDL_leng;

/* Points to current character in buffer. */
static char *__IDL__c_buf_p = (char *) 0;
static int __IDL__init = 1;		/* whether we need to initialize */
static int __IDL__start = 0;	/* start state number */

/* Flag which is used to allow __IDL_wrap()'s to do buffer switches
 * instead of setting up a fresh __IDL_in.  A bit of a hack ...
 */
static int __IDL__did_buffer_switch_on_eof;

void __IDL_restart YY_PROTO(( FILE *input_file ));

void __IDL__switch_to_buffer YY_PROTO(( YY_BUFFER_STATE new_buffer ));
void __IDL__load_buffer_state YY_PROTO(( void ));
YY_BUFFER_STATE __IDL__create_buffer YY_PROTO(( FILE *file, int size ));
void __IDL__delete_buffer YY_PROTO(( YY_BUFFER_STATE b ));
void __IDL__init_buffer YY_PROTO(( YY_BUFFER_STATE b, FILE *file ));
void __IDL__flush_buffer YY_PROTO(( YY_BUFFER_STATE b ));
#define YY_FLUSH_BUFFER __IDL__flush_buffer( __IDL__current_buffer )

YY_BUFFER_STATE __IDL__scan_buffer YY_PROTO(( char *base, __IDL__size_t size ));
YY_BUFFER_STATE __IDL__scan_string YY_PROTO(( __IDL_const char *__IDL__str ));
YY_BUFFER_STATE __IDL__scan_bytes YY_PROTO(( __IDL_const char *bytes, int len ));

static void *__IDL__flex_alloc YY_PROTO(( __IDL__size_t ));
static void *__IDL__flex_realloc YY_PROTO(( void *, __IDL__size_t ));
static void __IDL__flex_free YY_PROTO(( void * ));

#define __IDL__new_buffer __IDL__create_buffer

#define __IDL__set_interactive(is_interactive) \
	{ \
	if ( ! __IDL__current_buffer ) \
		__IDL__current_buffer = __IDL__create_buffer( __IDL_in, YY_BUF_SIZE ); \
	__IDL__current_buffer->__IDL__is_interactive = is_interactive; \
	}

#define __IDL__set_bol(at_bol) \
	{ \
	if ( ! __IDL__current_buffer ) \
		__IDL__current_buffer = __IDL__create_buffer( __IDL_in, YY_BUF_SIZE ); \
	__IDL__current_buffer->__IDL__at_bol = at_bol; \
	}

#define YY_AT_BOL() (__IDL__current_buffer->__IDL__at_bol)

typedef unsigned char YY_CHAR;
FILE *__IDL_in = (FILE *) 0, *__IDL_out = (FILE *) 0;
typedef int __IDL__state_type;
extern char *__IDL_text;
#define __IDL_text_ptr __IDL_text

static __IDL__state_type __IDL__get_previous_state YY_PROTO(( void ));
static __IDL__state_type __IDL__try_NUL_trans YY_PROTO(( __IDL__state_type current_state ));
static int __IDL__get_next_buffer YY_PROTO(( void ));
static void __IDL__fatal_error YY_PROTO(( __IDL_const char msg[] ));

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up __IDL_text.
 */
#define YY_DO_BEFORE_ACTION \
	__IDL_text_ptr = __IDL__bp; \
	__IDL_leng = (int) (__IDL__cp - __IDL__bp); \
	__IDL__hold_char = *__IDL__cp; \
	*__IDL__cp = '\0'; \
	__IDL__c_buf_p = __IDL__cp;

#define YY_NUM_RULES 69
#define YY_END_OF_BUFFER 70
static __IDL_const short int __IDL__accept[348] =
    {   0,
        6,    6,    6,    6,    6,    6,    6,    6,    6,    6,
        3,    3,   70,   68,    6,   65,   68,   68,   68,   68,
       68,    7,    9,   68,   68,   68,   56,   56,   56,   56,
       68,   56,   56,   56,   56,   56,   56,   56,   56,   56,
       56,   56,   56,   56,   56,   56,   56,   56,    6,    5,
       68,   68,   68,   60,   59,   68,    6,   65,   68,    6,
        5,    3,    3,    3,    3,    3,    3,    6,    0,   64,
        0,   63,    0,   12,    8,   12,   67,   66,   12,    7,
       11,    0,    0,    9,   52,   54,   53,   56,   58,   56,
       56,   56,   57,   57,   56,   56,   56,   56,   56,   56,

       56,   56,   56,   56,   56,   56,   29,   56,   56,   56,
       56,   56,   56,   56,   56,   56,   56,   56,   56,   56,
       56,   56,   56,   56,    6,    5,    5,    5,    5,    1,
        0,    0,   60,    0,   62,    6,   67,   66,    6,    5,
        5,    5,    5,    5,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    2,   12,    8,    0,   66,   12,    0,
       12,   10,   56,   56,   56,   57,   16,   56,   56,   56,
       56,   56,   56,   56,   56,   56,   56,   56,   56,   56,
       56,   56,   56,   56,   56,   37,   56,   56,   56,   56,
       56,   56,   56,   56,   56,   56,   56,   56,    5,    1,

       48,   61,   66,   62,    5,    3,    3,    2,    0,   12,
       56,   56,   15,   57,   56,   56,   19,   20,   56,   56,
       56,   56,   25,   56,   56,   56,   56,   56,   32,   56,
       56,   56,   56,   56,   56,   56,   56,   56,   56,   56,
       56,   56,   56,   49,   56,   56,    5,    5,    3,   13,
       56,   57,   56,   56,   21,   56,   56,   56,   56,   27,
       28,   30,   56,   56,   56,   35,   56,   56,   56,   56,
       41,   56,   56,   56,   56,   46,   56,   50,   56,    5,
        5,    3,   14,   57,   56,   56,   56,   56,   24,   56,
       56,   33,   34,   36,   38,   56,   56,   42,   43,   44,

       56,   56,   56,    5,    5,    3,   57,   56,   18,   22,
       23,   56,   56,   56,   56,   45,   56,   51,    4,    4,
        3,   57,   56,   56,   56,   39,   40,   47,    4,    4,
        4,    4,    4,    3,    3,   57,   17,   26,   31,   57,
        0,    0,    0,   55,    0,    0,    0
    } ;

static __IDL_const int __IDL__ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        2,    2,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    4,    5,    1,    6,    1,    7,    8,
        9,   10,    1,    1,   11,   12,   13,   14,   15,   15,
       15,   15,   15,   15,   15,   15,   15,   16,    1,   17,
        1,   18,    1,    1,   19,   20,   20,   21,   22,   23,
       24,   24,   24,   24,   24,   25,   24,   24,   26,   24,
       24,   27,   28,   29,   30,   24,   24,   31,   24,   24,
        1,    1,   32,    1,   33,    1,   34,   35,   36,   37,

       38,   39,   40,   41,   42,   43,   24,   44,   45,   46,
       47,   48,   49,   50,   51,   52,   53,   54,   55,   56,
       57,   24,   58,    1,   59,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static __IDL_const int __IDL__meta[60] =
    {   0,
        1,    2,    3,    1,    1,    1,    1,    1,    4,    1,
        1,    1,    1,    5,    5,    1,    1,    1,    6,    6,
        6,    6,    6,    7,    7,    7,    7,    7,    7,    7,
        7,    1,    8,    6,    6,    6,    6,    6,    6,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    1,    1
    } ;

static __IDL_const short int __IDL__base[373] =
    {   0,
        0,   58,  893,   60,   72,  131,   89,   94,    0,   65,
       98,  107,  904,  906,  901,  906,   65,  111,  107,   79,
      113,  179,  138,  886,  884,  882,  866,   96,   73,  124,
      865,  121,  122,  137,  139,  141,   83,  133,  135,  148,
      128,  152,  169,  171,  123,  150,  159,  178,  213,  197,
      839,  207,    0,    0,  906,  887,  115,  886,  215,  225,
      229,    0,  892,  223,  235,  206,  834,  890,  238,  906,
      236,  906,  232,  236,  241,  245,  906,    0,  250,  277,
      906,  270,    0,  281,  906,  906,  906,  858,    0,  219,
      237,  264,    0,  853,  243,  216,  228,  253,  272,  274,

      277,  268,  275,  276,  257,  278,  284,  289,  290,  291,
      293,  296,  297,  299,  304,  306,  300,  307,  309,  311,
      319,  320,  313,  315,  358,  348,    0,  354,  839,    0,
      876,  878,    0,  877,  906,  156,  876,  355,  363,  371,
      366,  374,    0,  375,    0,  882,    0,    0,  377,  364,
        0,  369,  833,    0,  373,  377,  386,    0,  406,  389,
      391,    0,  353,  375,  385,  844,  848,  376,  390,  391,
      383,  397,  398,  402,  405,  403,  407,  413,  406,  418,
      180,  409,  419,  420,  421,  847,  422,  427,  424,  430,
      432,  434,  433,  435,  436,  442,  450,  437,  845,    0,

      906,  906,  485,    0,  486,    0,  844,    0,  476,  478,
      474,  357,  844,  840,  439,  464,  842,  841,  465,  466,
      468,  467,  840,  470,  472,  473,  475,  479,  839,  480,
      477,  481,  482,  490,  489,  497,  486,  493,  501,  507,
      508,  509,  511,  838,  499,  514,  830,  543,  829,  835,
      515,  823,  524,  520,  833,  517,  525,  527,  528,  832,
      831,  830,  529,  533,  537,  829,  530,  531,  539,  543,
      828,  541,  544,  545,  546,  827,  555,  826,  557,  813,
      588,  810,  813,  779,  559,  560,  561,  562,  789,  565,
      566,  781,  765,  751,  743,  571,  569,  735,  708,  699,

      577,  584,  578,  646,  616,  645,  630,  575,  644,  643,
      639,  576,  588,  587,  593,  637,  595,  635,  658,  627,
      655,  611,  601,  602,  604,  614,  612,  610,    0,  639,
      637,  649,    0,    0,  636,  233,   95,   62,   28,  651,
      653,  660,  662,  906,  664,  672,  906,  681,  689,  697,
      705,  713,  717,  721,  729,  737,  741,  749,  757,  765,
      773,  781,  785,  787,  795,  803,  811,  819,  827,  835,
      843,  850
    } ;

static __IDL_const short int __IDL__def[373] =
    {   0,
      347,    1,    1,    1,  348,  348,  349,  349,    1,    1,
      350,  350,  347,  347,  347,  347,  351,  352,  347,  347,
      347,  347,  347,  347,  347,  347,  353,  353,  353,  353,
      354,  353,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  347,  355,
      347,  347,  356,  357,  347,  358,  358,  358,  358,  358,
      359,  360,  360,  360,  360,  361,  360,  347,  351,  347,
      352,  347,  347,  347,  347,  347,  347,  362,  347,  347,
      347,  347,  363,  347,  347,  347,  347,  353,  364,  353,
      353,  353,  354,  354,  353,  353,  353,  353,  353,  353,

      353,  353,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  347,  355,  355,  355,  355,  365,
      347,  356,  357,  358,  347,  358,  358,  366,  358,  359,
      359,  359,  355,  359,  360,  360,  360,  367,  360,  361,
      361,  361,  361,  368,  347,  347,  347,  362,  347,  347,
      347,  363,  353,  353,  353,  354,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  355,  365,

      347,  347,  366,  362,  359,  367,  361,  368,  347,  347,
      353,  353,  353,  354,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  355,  359,  361,  353,
      353,  354,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  353,  355,
      359,  361,  353,  354,  353,  353,  353,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  353,  353,

      353,  353,  353,  355,  359,  361,  354,  353,  353,  353,
      353,  353,  353,  353,  353,  353,  353,  353,  369,  370,
      371,  354,  353,  353,  353,  353,  353,  353,  369,  369,
      370,  370,  369,  371,  371,  354,  353,  353,  353,  354,
      347,  372,  372,  347,  372,  347,    0,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347
    } ;

static __IDL_const short int __IDL__nxt[966] =
    {   0,
       14,   15,   16,   17,   14,   14,   18,   14,   14,   14,
       19,   20,   21,   22,   23,   24,   25,   26,   27,   27,
       27,   27,   28,   27,   27,   29,   27,   27,   30,   27,
       27,   14,   31,   32,   33,   34,   35,   36,   37,   27,
       27,   38,   27,   39,   40,   41,   42,   27,   27,   43,
       44,   45,   46,   47,   48,   27,   27,   14,   14,   49,
       89,   49,   50,   51,   50,   51,   49,   70,   70,   50,
       51,   52,   14,   15,   16,   14,   14,   14,   14,   53,
       14,   14,   14,   14,   21,   14,   14,   14,   14,   14,
       57,   58,   76,   76,   89,   60,   58,   14,   61,   63,

       16,   59,   14,   55,   14,   89,   59,   91,   65,   16,
       64,   66,   67,   72,   90,   89,  136,   72,   73,   64,
       74,   75,   77,  135,  105,   78,  106,   89,   89,   14,
       14,   14,   49,   16,   14,   50,   14,   14,   53,   14,
       14,   14,   14,   21,   14,   14,   14,   14,   14,   79,
       92,   84,   84,   89,   89,   89,   89,  136,   81,   82,
       89,  110,   55,   14,  135,   89,   95,   89,   97,   89,
       98,   89,   96,   89,   81,   82,  101,   99,  107,  120,
       89,  108,   89,  100,   89,  102,  103,  111,   14,   14,
       79,   89,   80,   80,  109,  121,  104,  112,  128,   81,

       82,   89,  114,   89,  113,  122,  115,  152,  116,   83,
       89,  117,   89,  123,  125,   81,   82,  126,  131,  229,
       76,   76,  118,  135,  137,  119,  139,  138,  124,  140,
      142,  134,  147,  135,   83,  148,  149,  143,   72,  150,
       70,   70,   72,  163,  129,   76,   76,   79,   89,  155,
      155,   89,   79,  153,  156,  156,   81,   82,   76,   76,
       89,   81,   82,  159,  159,   81,  157,  168,  340,   89,
       81,   82,   81,   82,  169,   89,  144,   81,   82,  164,
      160,   81,  157,  161,  161,   89,   81,   82,   79,   89,
       80,   80,   79,  165,   84,   84,   89,   81,   82,  167,

       89,   81,   82,  170,   89,  171,   89,   89,   89,   89,
       89,  176,  177,   81,   82,  173,   89,   81,   82,  172,
      174,   89,   89,   89,  178,   89,  182,  175,   89,   89,
      179,   89,   89,  185,  181,  180,   89,  188,   89,   89,
      187,   89,  183,   89,  184,   89,  190,   89,  186,  128,
      192,   89,   89,  197,  189,  128,  191,  134,  193,  125,
      194,  196,  126,  204,  139,  152,  198,  140,  134,  195,
      152,  135,  142,  134,  143,  142,  134,  134,  149,  143,
      211,  150,  143,  143,   79,   89,  155,  155,   79,   89,
      156,  156,  251,   81,   82,  129,  209,   81,   82,  210,

      210,  129,  161,  161,  161,  161,  213,   89,   89,   81,
       82,  153,  212,   81,   82,   89,  153,   89,  144,  159,
      159,  144,   89,   89,  205,  215,   81,  157,  217,   89,
       89,  221,  218,  216,   89,   89,  222,   89,   89,   89,
      224,   89,   81,  157,  225,   89,  226,  219,  220,  223,
       89,   89,   89,   89,   89,  228,   89,  232,  227,   89,
      231,  230,   89,  235,   89,   89,   89,   89,   89,   89,
      241,   89,  234,  238,   89,  233,  236,  243,  244,  237,
      253,  242,   89,  245,  239,  240,  246,  134,  134,  210,
      210,  210,  210,  204,  143,  250,   89,   89,   89,   89,

       89,  254,   89,  256,   89,   89,   89,   89,  260,   89,
      258,   89,   89,   89,   89,  267,  255,  259,   89,  248,
      257,   89,   89,  264,  261,   89,  262,  268,  263,   89,
      265,   89,  266,   89,  270,  269,  273,  271,  272,   89,
       89,   89,  274,   89,  275,  134,   89,   89,  278,   89,
      277,  143,   89,  286,  276,  279,   89,   89,  285,   89,
       89,   89,   89,   89,  289,   89,  283,  291,  288,   89,
      292,   89,  287,   89,  293,   89,   89,   89,   89,  290,
      298,  295,  281,  301,  296,  300,  294,   89,  297,   89,
      134,   89,   89,   89,   89,  299,  143,   89,   89,  313,

      302,   89,  303,   89,  315,  309,  312,   89,   89,   89,
       89,  308,  310,  311,  314,  316,   89,  318,  134,   89,
       89,  317,  324,  325,  143,   89,  323,   89,  332,  134,
      327,  328,  305,   89,   89,  333,   89,  335,  337,  134,
      330,  339,   89,  326,   89,  333,   89,  338,  336,  320,
      332,  134,  341,  341,  341,  341,  335,  333,  342,  330,
      342,  343,  343,  343,  343,  346,  346,   89,  344,   89,
      344,   89,  344,  346,  346,   89,   89,  322,  321,  319,
      344,   54,   54,   54,   54,   54,   54,   54,   54,   56,
       56,   56,   56,   56,   56,   56,   56,   62,   62,   62,

       62,   62,   62,   62,   62,   69,   69,   69,   69,   69,
       69,   69,   69,   71,   71,   71,   71,   71,   71,   71,
       71,   88,   88,   88,   88,   93,   93,   93,   93,  127,
      127,   89,  127,  127,  127,  127,  127,  132,  132,  132,
       89,  132,  132,  132,  132,  133,  133,  133,  133,  134,
      134,  134,  134,  134,  134,  134,  134,  141,  141,  141,
      141,  141,  141,  141,  141,  145,  145,   89,  145,  145,
      145,  145,  145,  151,  151,   89,  151,  151,  151,  151,
      151,  158,  158,   89,  158,  158,  158,  158,  158,  162,
      162,   89,   89,   89,   89,  200,  200,   89,  200,  200,

      200,  200,  200,  203,  203,  203,  203,  203,  203,  203,
      203,  206,  206,   89,  206,  206,  206,  206,  206,  208,
      208,   89,  208,  208,  208,  208,  208,  329,  329,  307,
      329,  329,  329,  329,  329,  331,  331,  331,  331,  331,
      331,  331,  331,  334,  334,   89,  334,  334,  334,  334,
      334,  345,  345,  345,  306,  345,  345,  304,   89,   89,
       89,   89,   89,   89,   89,   89,  284,   89,  282,  280,
       89,   89,   89,   89,   89,  252,   89,  249,  247,   89,
       89,  214,  207,  146,  135,  135,  202,  201,  199,  166,
       89,   68,  154,  146,  135,  135,  130,   94,   89,   87,

       86,   85,   68,  347,   52,   13,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347
    } ;

static __IDL_const short int __IDL__chk[966] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    2,
      339,    4,    2,    2,    4,    4,   10,   17,   17,   10,
       10,    4,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        7,    7,   20,   20,  338,    8,    8,    7,    8,   11,

       11,    7,    8,    5,    5,   29,    8,   29,   12,   12,
       11,   12,   12,   18,   28,   37,   57,   18,   19,   12,
       19,   19,   21,   57,   37,   21,   37,  337,   28,    5,
        5,    6,    6,    6,    6,    6,    6,    6,    6,    6,
        6,    6,    6,    6,    6,    6,    6,    6,    6,   23,
       30,   23,   23,   32,   33,   45,   30,  136,   23,   23,
       41,   41,    6,    6,  136,   38,   32,   39,   33,   34,
       34,   35,   32,   36,   23,   23,   35,   34,   38,   45,
       40,   39,   46,   34,   42,   35,   36,   42,    6,    6,
       22,   47,   22,   22,   40,   46,   36,   42,   50,   22,

       22,   43,   43,   44,   42,   47,   43,   66,   44,   22,
       48,   44,  181,   48,   49,   22,   22,   49,   52,  181,
       52,   52,   44,   59,   59,   44,   60,   59,   48,   60,
       61,   61,   64,   60,   22,   64,   65,   61,   71,   65,
       69,   69,   71,   90,   50,   73,   73,   74,   96,   74,
       74,   90,   75,   66,   75,   75,   74,   74,   76,   76,
       97,   75,   75,   79,   79,   76,   76,   96,  336,   91,
       79,   79,   74,   74,   97,   95,   61,   75,   75,   91,
       82,   76,   76,   82,   82,   98,   79,   79,   80,  105,
       80,   80,   84,   92,   84,   84,   92,   80,   80,   95,

      102,   84,   84,   98,   99,   99,  100,  103,  104,  101,
      106,  104,  105,   80,   80,  101,  107,   84,   84,  100,
      102,  108,  109,  110,  106,  111,  109,  103,  112,  113,
      107,  114,  117,  112,  108,  107,  115,  115,  116,  118,
      114,  119,  110,  120,  111,  123,  117,  124,  113,  126,
      119,  121,  122,  123,  116,  128,  118,  138,  120,  125,
      121,  122,  125,  138,  139,  150,  124,  139,  141,  121,
      152,  139,  140,  140,  141,  142,  142,  144,  149,  140,
      163,  149,  142,  144,  155,  163,  155,  155,  156,  212,
      156,  156,  212,  155,  155,  126,  157,  156,  156,  157,

      157,  128,  160,  160,  161,  161,  165,  164,  168,  155,
      155,  150,  164,  156,  156,  171,  152,  165,  140,  159,
      159,  142,  169,  170,  144,  168,  159,  159,  170,  172,
      173,  173,  171,  169,  174,  176,  174,  175,  179,  177,
      176,  182,  159,  159,  177,  178,  178,  172,  172,  175,
      180,  183,  184,  185,  187,  180,  189,  184,  179,  188,
      183,  182,  190,  188,  191,  193,  192,  194,  195,  198,
      193,  215,  187,  191,  196,  185,  189,  195,  196,  190,
      215,  194,  197,  197,  191,  192,  198,  203,  205,  209,
      209,  210,  210,  203,  205,  211,  216,  219,  220,  222,

      221,  216,  224,  220,  225,  226,  211,  227,  225,  231,
      222,  228,  230,  232,  233,  233,  219,  224,  237,  205,
      221,  235,  234,  230,  226,  238,  227,  234,  228,  236,
      231,  245,  232,  239,  236,  235,  239,  237,  238,  240,
      241,  242,  240,  243,  241,  248,  246,  251,  245,  256,
      243,  248,  254,  254,  242,  246,  253,  257,  253,  258,
      259,  263,  267,  268,  258,  264,  251,  263,  257,  265,
      264,  269,  256,  272,  265,  270,  273,  274,  275,  259,
      272,  268,  248,  275,  269,  274,  267,  277,  270,  279,
      281,  285,  286,  287,  288,  273,  281,  290,  291,  291,

      277,  297,  279,  296,  297,  286,  290,  308,  312,  301,
      303,  285,  287,  288,  296,  301,  302,  303,  305,  314,
      313,  302,  312,  313,  305,  315,  308,  317,  320,  320,
      315,  317,  281,  323,  324,  320,  325,  335,  323,  331,
      330,  325,  328,  314,  327,  331,  326,  324,  322,  305,
      332,  332,  340,  340,  341,  341,  321,  332,  340,  319,
      341,  342,  342,  343,  343,  345,  345,  318,  342,  316,
      343,  311,  345,  346,  346,  310,  309,  307,  306,  304,
      346,  348,  348,  348,  348,  348,  348,  348,  348,  349,
      349,  349,  349,  349,  349,  349,  349,  350,  350,  350,

      350,  350,  350,  350,  350,  351,  351,  351,  351,  351,
      351,  351,  351,  352,  352,  352,  352,  352,  352,  352,
      352,  353,  353,  353,  353,  354,  354,  354,  354,  355,
      355,  300,  355,  355,  355,  355,  355,  356,  356,  356,
      299,  356,  356,  356,  356,  357,  357,  357,  357,  358,
      358,  358,  358,  358,  358,  358,  358,  359,  359,  359,
      359,  359,  359,  359,  359,  360,  360,  298,  360,  360,
      360,  360,  360,  361,  361,  295,  361,  361,  361,  361,
      361,  362,  362,  294,  362,  362,  362,  362,  362,  363,
      363,  364,  364,  364,  364,  365,  365,  293,  365,  365,

      365,  365,  365,  366,  366,  366,  366,  366,  366,  366,
      366,  367,  367,  292,  367,  367,  367,  367,  367,  368,
      368,  289,  368,  368,  368,  368,  368,  369,  369,  284,
      369,  369,  369,  369,  369,  370,  370,  370,  370,  370,
      370,  370,  370,  371,  371,  283,  371,  371,  371,  371,
      371,  372,  372,  372,  282,  372,  372,  280,  278,  276,
      271,  266,  262,  261,  260,  255,  252,  250,  249,  247,
      244,  229,  223,  218,  217,  214,  213,  207,  199,  186,
      167,  166,  153,  146,  137,  134,  132,  131,  129,   94,
       88,   68,   67,   63,   58,   56,   51,   31,   27,   26,

       25,   24,   15,   13,    3,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347,  347,  347,  347,  347,  347,
      347,  347,  347,  347,  347
    } ;

static __IDL__state_type __IDL__last_accepting_state;
static char *__IDL__last_accepting_cpos;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define __IDL_more() __IDL_more_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *__IDL_text;
#line 1 "./lexer.l"
#define INITIAL 0
/***************************************************************************

    lexer.l (IDL lex scanner)

    Copyright (C) 1998, 1999 Andrew T. Veliath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: lexer.l,v 1.71 1999/07/30 15:52:52 sopwith Exp $

***************************************************************************/
#line 25 "./lexer.l"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "rename.h"
#include "util.h"
#include "parser.h"

#ifdef XP_MAC
#  include <unix.h>
#  define YY_NEVER_INTERACTIVE 1
#endif

/* Eliminate warning */
#define YY_NO_UNPUT 1

#define YY_INPUT(buf,result,the_max_size)	do {				\
	if (__IDL_inputcb == NULL) {						\
		if ((result = fread (buf, 1, the_max_size, __IDL_in)) == YY_NULL &&	\
		    ferror (__IDL_in))						\
			YY_FATAL_ERROR ("input in scanner failed");		\
	} else {								\
		union IDL_input_data data;					\
										\
		data.fill.buffer = buf;						\
		data.fill.max_size = the_max_size;				\
		result = (*__IDL_inputcb) (IDL_INPUT_REASON_FILL, &data,	\
					   __IDL_inputcb_user_data);		\
		if (result < 0)							\
			YY_FATAL_ERROR ("input callback returned failure");	\
	}									\
} while (0)

#define tokreturn(token)			do {	\
	__IDL_prev_token_line = __IDL_cur_token_line;	\
	__IDL_cur_token_line = __IDL_cur_line;		\
	return token;					\
} while (0)

#define SELECT_START				\
	/* Parser driven start conditions */	\
	if (__IDL_flagsi & IDLFP_PROPERTIES)	\
		BEGIN (PROP);			\
	else if (__IDL_flagsi & IDLFP_NATIVE)	\
		BEGIN (NATIVE);			\
	/* Global syntax start conditions */	\
	else if (__IDL_flags & IDLF_XPIDL)	\
		BEGIN (XP);			\
	else if (__IDL_flags & IDLF_CODEFRAGS)	\
		BEGIN (CFRG);

#define SELECT_RESTART				\
	SELECT_START				\
	else					\
		BEGIN (INITIAL);

extern void		__IDL_do_pragma			(const char *s);
static int		count_nl			(const char *s);

#ifdef YYDEBUG
extern int				__IDL_debug;
#endif
int					__IDL_prev_token_line;
int					__IDL_cur_token_line;
static int				warn_underscores;
static char *				codefrag_desc;
static GSList *				codefrag_list;
static GSList *				codefrag_list_tail;
#define XP 1

#define PROP 2

#define NATIVE 3

#define CFRG 4

#define CFRGX 5


/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int __IDL_wrap YY_PROTO(( void ));
#else
extern int __IDL_wrap YY_PROTO(( void ));
#endif
#endif

#ifndef YY_NO_UNPUT
static void __IDL_unput YY_PROTO(( int c, char *buf_ptr ));
#endif

#ifndef __IDL_text_ptr
static void __IDL__flex_strncpy YY_PROTO(( char *, __IDL_const char *, int ));
#endif

#ifdef YY_NEED_STRLEN
static int __IDL__flex_strlen YY_PROTO(( __IDL_const char * ));
#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int __IDL_input YY_PROTO(( void ));
#else
static int input YY_PROTO(( void ));
#endif
#endif

#if YY_STACK_USED
static int __IDL__start_stack_ptr = 0;
static int __IDL__start_stack_depth = 0;
static int *__IDL__start_stack = 0;
#ifndef YY_NO_PUSH_STATE
static void __IDL__push_state YY_PROTO(( int new_state ));
#endif
#ifndef YY_NO_POP_STATE
static void __IDL__pop_state YY_PROTO(( void ));
#endif
#ifndef YY_NO_TOP_STATE
static int __IDL__top_state YY_PROTO(( void ));
#endif

#else
#define YY_NO_PUSH_STATE 1
#define YY_NO_POP_STATE 1
#define YY_NO_TOP_STATE 1
#endif

#ifdef YY_MALLOC_DECL
YY_MALLOC_DECL
#else
#if __STDC__
#ifndef __cplusplus
#include <stdlib.h>
#endif
#else
/* Just try to get by without declaring the routines.  This will fail
 * miserably on non-ANSI systems for which sizeof(size_t) != sizeof(int)
 * or sizeof(void*) != sizeof(int).
 */
#endif
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( __IDL_text, __IDL_leng, 1, __IDL_out )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( __IDL__current_buffer->__IDL__is_interactive ) \
		{ \
		int c = '*', n; \
		for ( n = 0; n < max_size && \
			     (c = getc( __IDL_in )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( __IDL_in ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else if ( ((result = fread( buf, 1, max_size, __IDL_in )) == 0) \
		  && ferror( __IDL_in ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "__IDL_terminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef __IDL_terminate
#define __IDL_terminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) __IDL__fatal_error( msg )
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL int __IDL_lex YY_PROTO(( void ))
#endif

/* Code executed at the beginning of each rule, after __IDL_text and __IDL_leng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	if ( __IDL_leng > 0 ) \
		__IDL__current_buffer->__IDL__at_bol = \
				(__IDL_text[__IDL_leng - 1] == '\n'); \
	YY_USER_ACTION

YY_DECL
	{
	register __IDL__state_type __IDL__current_state;
	register char *__IDL__cp, *__IDL__bp;
	register int __IDL__act;

#line 127 "./lexer.l"


	SELECT_START;


	if ( __IDL__init )
		{
		__IDL__init = 0;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! __IDL__start )
			__IDL__start = 1;	/* first start state */

		if ( ! __IDL_in )
			__IDL_in = stdin;

		if ( ! __IDL_out )
			__IDL_out = stdout;

		if ( ! __IDL__current_buffer )
			__IDL__current_buffer =
				__IDL__create_buffer( __IDL_in, YY_BUF_SIZE );

		__IDL__load_buffer_state();
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		__IDL__cp = __IDL__c_buf_p;

		/* Support of __IDL_text. */
		*__IDL__cp = __IDL__hold_char;

		/* __IDL__bp points to the position in __IDL__ch_buf of the start of
		 * the current run.
		 */
		__IDL__bp = __IDL__cp;

		__IDL__current_state = __IDL__start;
		__IDL__current_state += YY_AT_BOL();
__IDL__match:
		do
			{
			register YY_CHAR __IDL__c = __IDL__ec[YY_SC_TO_UI(*__IDL__cp)];
			if ( __IDL__accept[__IDL__current_state] )
				{
				__IDL__last_accepting_state = __IDL__current_state;
				__IDL__last_accepting_cpos = __IDL__cp;
				}
			while ( __IDL__chk[__IDL__base[__IDL__current_state] + __IDL__c] != __IDL__current_state )
				{
				__IDL__current_state = (int) __IDL__def[__IDL__current_state];
				if ( __IDL__current_state >= 348 )
					__IDL__c = __IDL__meta[(unsigned int) __IDL__c];
				}
			__IDL__current_state = __IDL__nxt[__IDL__base[__IDL__current_state] + (unsigned int) __IDL__c];
			++__IDL__cp;
			}
		while ( __IDL__base[__IDL__current_state] != 906 );

__IDL__find_action:
		__IDL__act = __IDL__accept[__IDL__current_state];
		if ( __IDL__act == 0 )
			{ /* have to back up */
			__IDL__cp = __IDL__last_accepting_cpos;
			__IDL__current_state = __IDL__last_accepting_state;
			__IDL__act = __IDL__accept[__IDL__current_state];
			}

		YY_DO_BEFORE_ACTION;


do_action:	/* This label is used only to access EOF actions. */


		switch ( __IDL__act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*__IDL__cp = __IDL__hold_char;
			__IDL__cp = __IDL__last_accepting_cpos;
			__IDL__current_state = __IDL__last_accepting_state;
			goto __IDL__find_action;

case 1:
YY_RULE_SETUP
#line 131 "./lexer.l"
{
	char *s = __IDL_text + 2;

	while (isspace (*s)) ++s;
	codefrag_desc = g_strdup (s);
	codefrag_list = codefrag_list_tail = NULL;

	if (!(__IDL_flags & IDLF_XPIDL || __IDL_flags & IDLF_CODEFRAGS))
		__IDL_error ("Code fragment syntax not enabled");
	else
		BEGIN (CFRGX);
}
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 143 "./lexer.l"
{
	__IDL_lval.tree = IDL_codefrag_new (codefrag_desc, codefrag_list);
	tokreturn (TOK_CODEFRAG);
}
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 147 "./lexer.l"
{
	char *s;
	GSList *slist;

	s = g_strdup (__IDL_text);
	slist = g_slist_alloc ();
	slist->data = s;

	if (codefrag_list == NULL) {
		codefrag_list = slist;
		codefrag_list_tail = slist;
	} else {
		codefrag_list_tail->next = slist;
		codefrag_list_tail = slist;
	}
}
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 163 "./lexer.l"
{
	int n;
	char *p = __IDL_text;
	char *s;

	while (isspace (*p) || *p == '#') ++p;
	s = p;
	sscanf (p, "%*6s%n", &n); s += n;
	while (isspace (*s)) ++s;

	__IDL_do_pragma (s);
}
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 175 "./lexer.l"
{
	char *starttext;
	char *filename;
	char *filename2;
	int line;

	starttext = __IDL_text;
	while (isspace (*starttext)) ++starttext;
	filename = g_malloc (strlen (starttext) + 1);

	sscanf (starttext, "# %d %s", &line, filename);

	if (*filename == '"') {
		filename2 = g_strdup (filename + 1);
		filename2[strlen (filename) - 2] = 0;
		g_free (filename);
	} else
		filename2 = filename;

	IDL_file_set (filename2, line);

	g_free (filename2);
}
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 198 "./lexer.l"
;
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 199 "./lexer.l"
{
	sscanf (__IDL_text, "%" IDL_LL "o", &__IDL_lval.integer);
	tokreturn (TOK_INTEGER);
}
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 203 "./lexer.l"
{
	sscanf (__IDL_text, "%" IDL_LL "d", &__IDL_lval.integer);
	tokreturn (TOK_INTEGER);
}
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 207 "./lexer.l"
{
	sscanf (__IDL_text, "%" IDL_LL "u", &__IDL_lval.integer);
	tokreturn (TOK_INTEGER);
}
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 211 "./lexer.l"
{
	sscanf (__IDL_text + 2, "%" IDL_LL "x", &__IDL_lval.integer);
	tokreturn (TOK_INTEGER);
}
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 215 "./lexer.l"
{
	__IDL_lval.str = g_strdup (__IDL_text);
	tokreturn (TOK_FIXEDP);
}
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 219 "./lexer.l"
{
	__IDL_lval.floatp = atof (__IDL_text);
	tokreturn (TOK_FLOATP);
}
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 223 "./lexer.l"
tokreturn (TOK_FALSE);
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 224 "./lexer.l"
tokreturn (TOK_OBJECT);
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 225 "./lexer.l"
tokreturn (TOK_TRUE);
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 226 "./lexer.l"
tokreturn (TOK_ANY);
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 227 "./lexer.l"
tokreturn (TOK_ATTRIBUTE);
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 228 "./lexer.l"
tokreturn (TOK_BOOLEAN);
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 229 "./lexer.l"
tokreturn (TOK_CASE);
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 230 "./lexer.l"
tokreturn (TOK_CHAR);
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 231 "./lexer.l"
tokreturn (TOK_CONST);
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 232 "./lexer.l"
tokreturn (TOK_CONTEXT);
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 233 "./lexer.l"
tokreturn (TOK_DEFAULT);
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 234 "./lexer.l"
tokreturn (TOK_DOUBLE);
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 235 "./lexer.l"
tokreturn (TOK_ENUM);
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 236 "./lexer.l"
tokreturn (TOK_EXCEPTION);
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 237 "./lexer.l"
tokreturn (TOK_FIXED);
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 238 "./lexer.l"
tokreturn (TOK_FLOAT);
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 239 "./lexer.l"
tokreturn (TOK_IN);
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 240 "./lexer.l"
tokreturn (TOK_INOUT);
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 241 "./lexer.l"
tokreturn (TOK_INTERFACE);
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 242 "./lexer.l"
tokreturn (TOK_LONG);
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 243 "./lexer.l"
tokreturn (TOK_MODULE);
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 244 "./lexer.l"
tokreturn (TOK_NATIVE);
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 245 "./lexer.l"
tokreturn (TOK_OCTET);
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 246 "./lexer.l"
tokreturn (TOK_ONEWAY);
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 247 "./lexer.l"
tokreturn (TOK_OUT);
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 248 "./lexer.l"
tokreturn (TOK_RAISES);
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 249 "./lexer.l"
tokreturn (TOK_READONLY);
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 250 "./lexer.l"
tokreturn (TOK_SEQUENCE);
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 251 "./lexer.l"
tokreturn (TOK_SHORT);
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 252 "./lexer.l"
tokreturn (TOK_STRING);
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 253 "./lexer.l"
tokreturn (TOK_STRUCT);
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 254 "./lexer.l"
tokreturn (TOK_SWITCH);
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 255 "./lexer.l"
tokreturn (TOK_TYPEDEF);
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 256 "./lexer.l"
tokreturn (TOK_UNION);
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 257 "./lexer.l"
tokreturn (TOK_UNSIGNED);
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 258 "./lexer.l"
tokreturn (TOK_VARARGS);
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 259 "./lexer.l"
tokreturn (TOK_VOID);
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 260 "./lexer.l"
tokreturn (TOK_WCHAR);
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 261 "./lexer.l"
tokreturn (TOK_WSTRING);
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 262 "./lexer.l"
tokreturn (TOK_OP_SCOPE);
	YY_BREAK
case 53:
YY_RULE_SETUP
#line 263 "./lexer.l"
tokreturn (TOK_OP_SHR);
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 264 "./lexer.l"
tokreturn (TOK_OP_SHL);
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 265 "./lexer.l"
{
	char *s = g_strdup (__IDL_text);

	/* Get the parenthesized expression (ignoring whitespace) */
	sscanf (__IDL_text, "__declspec %*[(] %[A-Za-z_] %*[)]", s);
	__IDL_lval.str = s;
	__IDL_cur_line += count_nl (__IDL_text);
	tokreturn (TOK_DECLSPEC);
}
	YY_BREAK
case 56:
YY_RULE_SETUP
#line 274 "./lexer.l"
{
	if (__IDL_flags & IDLF_TYPECODES && strcmp (__IDL_text, "TypeCode") == 0)
		tokreturn (TOK_TYPECODE);
	__IDL_lval.str = g_strdup (__IDL_text);
	tokreturn (TOK_IDENT);
}
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 280 "./lexer.l"
{
	__IDL_errorv ("`%s' is not a valid identifier in IDL", __IDL_text);
	__IDL_error ("(Identifiers cannot start with an underscore)");
	__IDL_lval.str = g_strdup (__IDL_text);
	tokreturn (TOK_IDENT);
}
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 286 "./lexer.l"
{
	if (!warn_underscores) {
		__IDL_warningv (IDL_WARNING2,
			   "`%s' underscores within identifiers are discouraged for use "
			   "with C-language IDL mappings", __IDL_text);
		warn_underscores = 1;
	}
	__IDL_lval.str = g_strdup (__IDL_text);
	tokreturn (TOK_IDENT);
}
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 296 "./lexer.l"
{
	__IDL_flagsi &= ~IDLFP_PROPERTIES;
	SELECT_RESTART;
	tokreturn (__IDL_text[0]);
}
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 301 "./lexer.l"
{
	__IDL_lval.str = g_strdup (__IDL_text);
	tokreturn (TOK_PROP_KEY);
}
	YY_BREAK
case 61:
YY_RULE_SETUP
#line 305 "./lexer.l"
{
	__IDL_lval.str = g_strdup (__IDL_text + 1);
	__IDL_lval.str[strlen (__IDL_lval.str) - 1] = 0;
	tokreturn (TOK_PROP_VALUE);
}
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 310 "./lexer.l"
{
	__IDL_flagsi &= ~IDLFP_NATIVE;
	__IDL_lval.str = g_strdup (__IDL_text);
	__IDL_lval.str[strlen (__IDL_lval.str) - 1] = 0;
	tokreturn (TOK_NATIVE_TYPE);
}
	YY_BREAK
case 63:
YY_RULE_SETUP
#line 316 "./lexer.l"
{
	__IDL_lval.str = g_strdup (__IDL_text + 1);
	__IDL_lval.str[strlen (__IDL_text) - 2] = 0;
	tokreturn (TOK_SQSTRING);
}
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 321 "./lexer.l"
{
	__IDL_lval.str = g_strdup (__IDL_text + 1);
	__IDL_lval.str[strlen (__IDL_text) - 2] = 0;
	tokreturn (TOK_DQSTRING);
}
	YY_BREAK
case 65:
YY_RULE_SETUP
#line 326 "./lexer.l"
++__IDL_cur_line;
	YY_BREAK
case 66:
YY_RULE_SETUP
#line 327 "./lexer.l"
;
	YY_BREAK
case 67:
YY_RULE_SETUP
#line 328 "./lexer.l"
{
	int c;

	while (1) {
		while ((c = input ()) != '*' && c != EOF)
			if (c == '\n') ++__IDL_cur_line;
		if (c == '*') {
			while ((c = input ()) == '*') ;
			if (c == '/') break;
		}
		if (c == '\n') ++__IDL_cur_line;
		if (c == EOF) {
			__IDL_warning (IDL_WARNING1, "End of file in comment");
			break;
		}
	}
}
	YY_BREAK
case 68:
YY_RULE_SETUP
#line 345 "./lexer.l"
tokreturn (__IDL_text[0]);
	YY_BREAK
case 69:
YY_RULE_SETUP
#line 347 "./lexer.l"
ECHO;
	YY_BREAK
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(XP):
case YY_STATE_EOF(PROP):
case YY_STATE_EOF(NATIVE):
case YY_STATE_EOF(CFRG):
case YY_STATE_EOF(CFRGX):
	__IDL_terminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int __IDL__amount_of_matched_text = (int) (__IDL__cp - __IDL_text_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*__IDL__cp = __IDL__hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( __IDL__current_buffer->__IDL__buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed __IDL_in at a new source and called
			 * __IDL_lex().  If so, then we have to assure
			 * consistency between __IDL__current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			__IDL__n_chars = __IDL__current_buffer->__IDL__n_chars;
			__IDL__current_buffer->__IDL__input_file = __IDL_in;
			__IDL__current_buffer->__IDL__buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for __IDL__c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since __IDL__c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( __IDL__c_buf_p <= &__IDL__current_buffer->__IDL__ch_buf[__IDL__n_chars] )
			{ /* This was really a NUL. */
			__IDL__state_type __IDL__next_state;

			__IDL__c_buf_p = __IDL_text_ptr + __IDL__amount_of_matched_text;

			__IDL__current_state = __IDL__get_previous_state();

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * __IDL__get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			__IDL__next_state = __IDL__try_NUL_trans( __IDL__current_state );

			__IDL__bp = __IDL_text_ptr + YY_MORE_ADJ;

			if ( __IDL__next_state )
				{
				/* Consume the NUL. */
				__IDL__cp = ++__IDL__c_buf_p;
				__IDL__current_state = __IDL__next_state;
				goto __IDL__match;
				}

			else
				{
				__IDL__cp = __IDL__c_buf_p;
				goto __IDL__find_action;
				}
			}

		else switch ( __IDL__get_next_buffer() )
			{
			case EOB_ACT_END_OF_FILE:
				{
				__IDL__did_buffer_switch_on_eof = 0;

				if ( __IDL_wrap() )
					{
					/* Note: because we've taken care in
					 * __IDL__get_next_buffer() to have set up
					 * __IDL_text, we can now set up
					 * __IDL__c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					__IDL__c_buf_p = __IDL_text_ptr + YY_MORE_ADJ;

					__IDL__act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! __IDL__did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				__IDL__c_buf_p =
					__IDL_text_ptr + __IDL__amount_of_matched_text;

				__IDL__current_state = __IDL__get_previous_state();

				__IDL__cp = __IDL__c_buf_p;
				__IDL__bp = __IDL_text_ptr + YY_MORE_ADJ;
				goto __IDL__match;

			case EOB_ACT_LAST_MATCH:
				__IDL__c_buf_p =
				&__IDL__current_buffer->__IDL__ch_buf[__IDL__n_chars];

				__IDL__current_state = __IDL__get_previous_state();

				__IDL__cp = __IDL__c_buf_p;
				__IDL__bp = __IDL_text_ptr + YY_MORE_ADJ;
				goto __IDL__find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of __IDL_lex */


/* __IDL__get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

static int __IDL__get_next_buffer()
	{
	register char *dest = __IDL__current_buffer->__IDL__ch_buf;
	register char *source = __IDL_text_ptr;
	register int number_to_move, i;
	int ret_val;

	if ( __IDL__c_buf_p > &__IDL__current_buffer->__IDL__ch_buf[__IDL__n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( __IDL__current_buffer->__IDL__fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( __IDL__c_buf_p - __IDL_text_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (__IDL__c_buf_p - __IDL_text_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( __IDL__current_buffer->__IDL__buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		__IDL__current_buffer->__IDL__n_chars = __IDL__n_chars = 0;

	else
		{
		int num_to_read =
			__IDL__current_buffer->__IDL__buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = __IDL__current_buffer;

			int __IDL__c_buf_p_offset =
				(int) (__IDL__c_buf_p - b->__IDL__ch_buf);

			if ( b->__IDL__is_our_buffer )
				{
				int new_size = b->__IDL__buf_size * 2;

				if ( new_size <= 0 )
					b->__IDL__buf_size += b->__IDL__buf_size / 8;
				else
					b->__IDL__buf_size *= 2;

				b->__IDL__ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					__IDL__flex_realloc( (void *) b->__IDL__ch_buf,
							 b->__IDL__buf_size + 2 );
				}
			else
				/* Can't grow it, we don't own it. */
				b->__IDL__ch_buf = 0;

			if ( ! b->__IDL__ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			__IDL__c_buf_p = &b->__IDL__ch_buf[__IDL__c_buf_p_offset];

			num_to_read = __IDL__current_buffer->__IDL__buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&__IDL__current_buffer->__IDL__ch_buf[number_to_move]),
			__IDL__n_chars, num_to_read );

		__IDL__current_buffer->__IDL__n_chars = __IDL__n_chars;
		}

	if ( __IDL__n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			__IDL_restart( __IDL_in );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			__IDL__current_buffer->__IDL__buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	__IDL__n_chars += number_to_move;
	__IDL__current_buffer->__IDL__ch_buf[__IDL__n_chars] = YY_END_OF_BUFFER_CHAR;
	__IDL__current_buffer->__IDL__ch_buf[__IDL__n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	__IDL_text_ptr = &__IDL__current_buffer->__IDL__ch_buf[0];

	return ret_val;
	}


/* __IDL__get_previous_state - get the state just before the EOB char was reached */

static __IDL__state_type __IDL__get_previous_state()
	{
	register __IDL__state_type __IDL__current_state;
	register char *__IDL__cp;

	__IDL__current_state = __IDL__start;
	__IDL__current_state += YY_AT_BOL();

	for ( __IDL__cp = __IDL_text_ptr + YY_MORE_ADJ; __IDL__cp < __IDL__c_buf_p; ++__IDL__cp )
		{
		register YY_CHAR __IDL__c = (*__IDL__cp ? __IDL__ec[YY_SC_TO_UI(*__IDL__cp)] : 1);
		if ( __IDL__accept[__IDL__current_state] )
			{
			__IDL__last_accepting_state = __IDL__current_state;
			__IDL__last_accepting_cpos = __IDL__cp;
			}
		while ( __IDL__chk[__IDL__base[__IDL__current_state] + __IDL__c] != __IDL__current_state )
			{
			__IDL__current_state = (int) __IDL__def[__IDL__current_state];
			if ( __IDL__current_state >= 348 )
				__IDL__c = __IDL__meta[(unsigned int) __IDL__c];
			}
		__IDL__current_state = __IDL__nxt[__IDL__base[__IDL__current_state] + (unsigned int) __IDL__c];
		}

	return __IDL__current_state;
	}


/* __IDL__try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = __IDL__try_NUL_trans( current_state );
 */

#ifdef YY_USE_PROTOS
static __IDL__state_type __IDL__try_NUL_trans( __IDL__state_type __IDL__current_state )
#else
static __IDL__state_type __IDL__try_NUL_trans( __IDL__current_state )
__IDL__state_type __IDL__current_state;
#endif
	{
	register int __IDL__is_jam;
	register char *__IDL__cp = __IDL__c_buf_p;

	register YY_CHAR __IDL__c = 1;
	if ( __IDL__accept[__IDL__current_state] )
		{
		__IDL__last_accepting_state = __IDL__current_state;
		__IDL__last_accepting_cpos = __IDL__cp;
		}
	while ( __IDL__chk[__IDL__base[__IDL__current_state] + __IDL__c] != __IDL__current_state )
		{
		__IDL__current_state = (int) __IDL__def[__IDL__current_state];
		if ( __IDL__current_state >= 348 )
			__IDL__c = __IDL__meta[(unsigned int) __IDL__c];
		}
	__IDL__current_state = __IDL__nxt[__IDL__base[__IDL__current_state] + (unsigned int) __IDL__c];
	__IDL__is_jam = (__IDL__current_state == 347);

	return __IDL__is_jam ? 0 : __IDL__current_state;
	}


#ifndef YY_NO_UNPUT
#ifdef YY_USE_PROTOS
static void __IDL_unput( int c, register char *__IDL__bp )
#else
static void __IDL_unput( c, __IDL__bp )
int c;
register char *__IDL__bp;
#endif
	{
	register char *__IDL__cp = __IDL__c_buf_p;

	/* undo effects of setting up __IDL_text */
	*__IDL__cp = __IDL__hold_char;

	if ( __IDL__cp < __IDL__current_buffer->__IDL__ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = __IDL__n_chars + 2;
		register char *dest = &__IDL__current_buffer->__IDL__ch_buf[
					__IDL__current_buffer->__IDL__buf_size + 2];
		register char *source =
				&__IDL__current_buffer->__IDL__ch_buf[number_to_move];

		while ( source > __IDL__current_buffer->__IDL__ch_buf )
			*--dest = *--source;

		__IDL__cp += (int) (dest - source);
		__IDL__bp += (int) (dest - source);
		__IDL__current_buffer->__IDL__n_chars =
			__IDL__n_chars = __IDL__current_buffer->__IDL__buf_size;

		if ( __IDL__cp < __IDL__current_buffer->__IDL__ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--__IDL__cp = (char) c;


	__IDL_text_ptr = __IDL__bp;
	__IDL__hold_char = *__IDL__cp;
	__IDL__c_buf_p = __IDL__cp;
	}
#endif	/* ifndef YY_NO_UNPUT */


#ifdef __cplusplus
static int __IDL_input()
#else
static int input()
#endif
	{
	int c;

	*__IDL__c_buf_p = __IDL__hold_char;

	if ( *__IDL__c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* __IDL__c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( __IDL__c_buf_p < &__IDL__current_buffer->__IDL__ch_buf[__IDL__n_chars] )
			/* This was really a NUL. */
			*__IDL__c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = __IDL__c_buf_p - __IDL_text_ptr;
			++__IDL__c_buf_p;

			switch ( __IDL__get_next_buffer() )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because __IDL__g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					__IDL_restart( __IDL_in );

					/* fall through */

				case EOB_ACT_END_OF_FILE:
					{
					if ( __IDL_wrap() )
						return EOF;

					if ( ! __IDL__did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return __IDL_input();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					__IDL__c_buf_p = __IDL_text_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) __IDL__c_buf_p;	/* cast for 8-bit char's */
	*__IDL__c_buf_p = '\0';	/* preserve __IDL_text */
	__IDL__hold_char = *++__IDL__c_buf_p;

	__IDL__current_buffer->__IDL__at_bol = (c == '\n');

	return c;
	}


#ifdef YY_USE_PROTOS
void __IDL_restart( FILE *input_file )
#else
void __IDL_restart( input_file )
FILE *input_file;
#endif
	{
	if ( ! __IDL__current_buffer )
		__IDL__current_buffer = __IDL__create_buffer( __IDL_in, YY_BUF_SIZE );

	__IDL__init_buffer( __IDL__current_buffer, input_file );
	__IDL__load_buffer_state();
	}


#ifdef YY_USE_PROTOS
void __IDL__switch_to_buffer( YY_BUFFER_STATE new_buffer )
#else
void __IDL__switch_to_buffer( new_buffer )
YY_BUFFER_STATE new_buffer;
#endif
	{
	if ( __IDL__current_buffer == new_buffer )
		return;

	if ( __IDL__current_buffer )
		{
		/* Flush out information for old buffer. */
		*__IDL__c_buf_p = __IDL__hold_char;
		__IDL__current_buffer->__IDL__buf_pos = __IDL__c_buf_p;
		__IDL__current_buffer->__IDL__n_chars = __IDL__n_chars;
		}

	__IDL__current_buffer = new_buffer;
	__IDL__load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (__IDL_wrap()) processing, but the only time this flag
	 * is looked at is after __IDL_wrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	__IDL__did_buffer_switch_on_eof = 1;
	}


#ifdef YY_USE_PROTOS
void __IDL__load_buffer_state( void )
#else
void __IDL__load_buffer_state()
#endif
	{
	__IDL__n_chars = __IDL__current_buffer->__IDL__n_chars;
	__IDL_text_ptr = __IDL__c_buf_p = __IDL__current_buffer->__IDL__buf_pos;
	__IDL_in = __IDL__current_buffer->__IDL__input_file;
	__IDL__hold_char = *__IDL__c_buf_p;
	}


#ifdef YY_USE_PROTOS
YY_BUFFER_STATE __IDL__create_buffer( FILE *file, int size )
#else
YY_BUFFER_STATE __IDL__create_buffer( file, size )
FILE *file;
int size;
#endif
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) __IDL__flex_alloc( sizeof( struct __IDL__buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in __IDL__create_buffer()" );

	b->__IDL__buf_size = size;

	/* __IDL__ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->__IDL__ch_buf = (char *) __IDL__flex_alloc( b->__IDL__buf_size + 2 );
	if ( ! b->__IDL__ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in __IDL__create_buffer()" );

	b->__IDL__is_our_buffer = 1;

	__IDL__init_buffer( b, file );

	return b;
	}


#ifdef YY_USE_PROTOS
void __IDL__delete_buffer( YY_BUFFER_STATE b )
#else
void __IDL__delete_buffer( b )
YY_BUFFER_STATE b;
#endif
	{
	if ( ! b )
		return;

	if ( b == __IDL__current_buffer )
		__IDL__current_buffer = (YY_BUFFER_STATE) 0;

	if ( b->__IDL__is_our_buffer )
		__IDL__flex_free( (void *) b->__IDL__ch_buf );

	__IDL__flex_free( (void *) b );
	}


#ifndef YY_ALWAYS_INTERACTIVE
#ifndef YY_NEVER_INTERACTIVE
extern int isatty YY_PROTO(( int ));
#endif
#endif

#ifdef YY_USE_PROTOS
void __IDL__init_buffer( YY_BUFFER_STATE b, FILE *file )
#else
void __IDL__init_buffer( b, file )
YY_BUFFER_STATE b;
FILE *file;
#endif


	{
	__IDL__flush_buffer( b );

	b->__IDL__input_file = file;
	b->__IDL__fill_buffer = 1;

#if YY_ALWAYS_INTERACTIVE
	b->__IDL__is_interactive = 1;
#else
#if YY_NEVER_INTERACTIVE
	b->__IDL__is_interactive = 0;
#else
	b->__IDL__is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
#endif
#endif
	}


#ifdef YY_USE_PROTOS
void __IDL__flush_buffer( YY_BUFFER_STATE b )
#else
void __IDL__flush_buffer( b )
YY_BUFFER_STATE b;
#endif

	{
	if ( ! b )
		return;

	b->__IDL__n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->__IDL__ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->__IDL__ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->__IDL__buf_pos = &b->__IDL__ch_buf[0];

	b->__IDL__at_bol = 1;
	b->__IDL__buffer_status = YY_BUFFER_NEW;

	if ( b == __IDL__current_buffer )
		__IDL__load_buffer_state();
	}


#ifndef YY_NO_SCAN_BUFFER
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE __IDL__scan_buffer( char *base, __IDL__size_t size )
#else
YY_BUFFER_STATE __IDL__scan_buffer( base, size )
char *base;
__IDL__size_t size;
#endif
	{
	YY_BUFFER_STATE b;

	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) __IDL__flex_alloc( sizeof( struct __IDL__buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in __IDL__scan_buffer()" );

	b->__IDL__buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->__IDL__buf_pos = b->__IDL__ch_buf = base;
	b->__IDL__is_our_buffer = 0;
	b->__IDL__input_file = 0;
	b->__IDL__n_chars = b->__IDL__buf_size;
	b->__IDL__is_interactive = 0;
	b->__IDL__at_bol = 1;
	b->__IDL__fill_buffer = 0;
	b->__IDL__buffer_status = YY_BUFFER_NEW;

	__IDL__switch_to_buffer( b );

	return b;
	}
#endif


#ifndef YY_NO_SCAN_STRING
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE __IDL__scan_string( __IDL_const char *__IDL__str )
#else
YY_BUFFER_STATE __IDL__scan_string( __IDL__str )
__IDL_const char *__IDL__str;
#endif
	{
	int len;
	for ( len = 0; __IDL__str[len]; ++len )
		;

	return __IDL__scan_bytes( __IDL__str, len );
	}
#endif


#ifndef YY_NO_SCAN_BYTES
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE __IDL__scan_bytes( __IDL_const char *bytes, int len )
#else
YY_BUFFER_STATE __IDL__scan_bytes( bytes, len )
__IDL_const char *bytes;
int len;
#endif
	{
	YY_BUFFER_STATE b;
	char *buf;
	__IDL__size_t n;
	int i;

	/* Get memory for full buffer, including space for trailing EOB's. */
	n = len + 2;
	buf = (char *) __IDL__flex_alloc( n );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in __IDL__scan_bytes()" );

	for ( i = 0; i < len; ++i )
		buf[i] = bytes[i];

	buf[len] = buf[len+1] = YY_END_OF_BUFFER_CHAR;

	b = __IDL__scan_buffer( buf, n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in __IDL__scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->__IDL__is_our_buffer = 1;

	return b;
	}
#endif


#ifndef YY_NO_PUSH_STATE
#ifdef YY_USE_PROTOS
static void __IDL__push_state( int new_state )
#else
static void __IDL__push_state( new_state )
int new_state;
#endif
	{
	if ( __IDL__start_stack_ptr >= __IDL__start_stack_depth )
		{
		__IDL__size_t new_size;

		__IDL__start_stack_depth += YY_START_STACK_INCR;
		new_size = __IDL__start_stack_depth * sizeof( int );

		if ( ! __IDL__start_stack )
			__IDL__start_stack = (int *) __IDL__flex_alloc( new_size );

		else
			__IDL__start_stack = (int *) __IDL__flex_realloc(
					(void *) __IDL__start_stack, new_size );

		if ( ! __IDL__start_stack )
			YY_FATAL_ERROR(
			"out of memory expanding start-condition stack" );
		}

	__IDL__start_stack[__IDL__start_stack_ptr++] = YY_START;

	BEGIN(new_state);
	}
#endif


#ifndef YY_NO_POP_STATE
static void __IDL__pop_state()
	{
	if ( --__IDL__start_stack_ptr < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(__IDL__start_stack[__IDL__start_stack_ptr]);
	}
#endif


#ifndef YY_NO_TOP_STATE
static int __IDL__top_state()
	{
	return __IDL__start_stack[__IDL__start_stack_ptr - 1];
	}
#endif

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

#ifdef YY_USE_PROTOS
static void __IDL__fatal_error( __IDL_const char msg[] )
#else
static void __IDL__fatal_error( msg )
char msg[];
#endif
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
	}



/* Redefine __IDL_less() so it works in section 3 code. */

#undef __IDL_less
#define __IDL_less(n) \
	do \
		{ \
		/* Undo effects of setting up __IDL_text. */ \
		__IDL_text[__IDL_leng] = __IDL__hold_char; \
		__IDL__c_buf_p = __IDL_text + n; \
		__IDL__hold_char = *__IDL__c_buf_p; \
		*__IDL__c_buf_p = '\0'; \
		__IDL_leng = n; \
		} \
	while ( 0 )


/* Internal utility routines. */

#ifndef __IDL_text_ptr
#ifdef YY_USE_PROTOS
static void __IDL__flex_strncpy( char *s1, __IDL_const char *s2, int n )
#else
static void __IDL__flex_strncpy( s1, s2, n )
char *s1;
__IDL_const char *s2;
int n;
#endif
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif

#ifdef YY_NEED_STRLEN
#ifdef YY_USE_PROTOS
static int __IDL__flex_strlen( __IDL_const char *s )
#else
static int __IDL__flex_strlen( s )
__IDL_const char *s;
#endif
	{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
	}
#endif


#ifdef YY_USE_PROTOS
static void *__IDL__flex_alloc( __IDL__size_t size )
#else
static void *__IDL__flex_alloc( size )
__IDL__size_t size;
#endif
	{
	return (void *) malloc( size );
	}

#ifdef YY_USE_PROTOS
static void *__IDL__flex_realloc( void *ptr, __IDL__size_t size )
#else
static void *__IDL__flex_realloc( ptr, size )
void *ptr;
__IDL__size_t size;
#endif
	{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
	}

#ifdef YY_USE_PROTOS
static void __IDL__flex_free( void *ptr )
#else
static void __IDL__flex_free( ptr )
void *ptr;
#endif
	{
	free( ptr );
	}

#if YY_MAIN
int main()
	{
	__IDL_lex();
	return 0;
	}
#endif
#line 347 "./lexer.l"


void __IDL_lex_init (void)
{
	__IDL_inputcb = NULL;
	__IDL_cur_line = 1;
	__IDL_cur_token_line = 0;
	__IDL_prev_token_line = 0;
	__IDL_cur_filename = NULL;
	__IDL_cur_fileinfo = NULL;
	warn_underscores = 0;
}

void __IDL_lex_cleanup (void)
{
	__IDL_cur_filename = NULL;
	YY_NEW_FILE;
}

int __IDL_wrap (void)
{
	return 1;
}

static int count_nl (const char *s)
{
	int i;

	for (i = 0;
	     (s = strchr (s, '\n')) != NULL;
	     ++s, ++i) ;

	return i;
}

/*
 * Local variables:
 * mode: C
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: t
 * End:
 */
