#ifndef __RENAME_H
#define __RENAME_H

#define	yymaxdepth 			__IDL_maxdepth
#define	yyparse				__IDL_parse
#define	yylex				__IDL_lex
#define	yylval				__IDL_lval
#define	yychar				__IDL_char
#define	yydebug				__IDL_debug
#define	yypact				__IDL_pact
#define	yyr1				__IDL_r1
#define	yyr2				__IDL_r2
#define	yydef				__IDL_def
#define	yychk				__IDL_chk
#define	yypgo				__IDL_pgo
#define	yyact				__IDL_act
#define	yyexca				__IDL_exca
#define yyerrflag 			__IDL_errflag
#define yynerrs				__IDL_nerrs
#define	yyps				__IDL_ps
#define	yypv				__IDL_pv
#define	yys				__IDL_s
#define	yy_yys				__IDL_yys
#define	yystate				__IDL_state
#define	yytmp				__IDL_tmp
#define	yyv				__IDL_v
#define	yy_yyv				__IDL_yyv
#define	yyval				__IDL_val
#define	yylloc				__IDL_lloc
#define yyreds				__IDL_reds
#define yytoks				__IDL_toks
#define yylhs				__IDL_lhs
#define yylen				__IDL_len
#define yydefred 			__IDL_defred
#define yydgoto				__IDL_dgoto
#define yysindex 			__IDL_sindex
#define yyrindex 			__IDL_rindex
#define yygindex 			__IDL_gindex
#define yytable	 			__IDL_table
#define yycheck	 			__IDL_check
#define	yyerror				__IDL_error
#define yywarning			__IDL_warning
#define	yyerrorl			__IDL_errorl
#define yywarningl			__IDL_warningl
#define	yyerrorv			__IDL_errorv
#define yywarningv			__IDL_warningv
#define	yyerrorlv			__IDL_errorlv
#define yywarninglv			__IDL_warninglv

#endif /* __RENAME_H */
